#!/usr/bin/env python
################################# LICENSE ##################################
# Copyright (c) 2009, South African Astronomical Observatory (SAAO)        #
# All rights reserved.                                                     #
#                                                                          #
# Redistribution and use in source and binary forms, with or without       #
# modification, are permitted provided that the following conditions       #
# are met:                                                                 #
#                                                                          #
#     * Redistributions of source code must retain the above copyright     #
#       notice, this list of conditions and the following disclaimer.      #
#     * Redistributions in binary form must reproduce the above copyright  #
#       notice, this list of conditions and the following disclaimer       #
#       in the documentation and/or other materials provided with the      #
#       distribution.                                                      #
#     * Neither the name of the South African Astronomical Observatory     #
#       (SAAO) nor the names of its contributors may be used to endorse    #
#       or promote products derived from this software without specific    #
#       prior written permission.                                          #
#                                                                          #
# THIS SOFTWARE IS PROVIDED BY THE SAAO ''AS IS'' AND ANY EXPRESS OR       #
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           #
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   #
# DISCLAIMED. IN NO EVENT SHALL THE SAAO BE LIABLE FOR ANY                 #
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       #
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS  #
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)    #
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,      #
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          #
# POSSIBILITY OF SUCH DAMAGE.                                              #
############################################################################
"""
SPECIDENTIFY  is a program to read in SALT RSS spectroscopic arc lamps and 
determine the wavelength solution for that data.  The input data should be
a SALT arc lamp and a line list or another arc lamp image with high quality
wavelength solution. The lamp list can be either wavelengths, wavelengths
and fluxes, or an arc image with a high quality solution.  The line lamp
can also be left unspecified and the user will manually enter the data.

From there, the user has several different possible choices.  They can provide
a first guess of the coefficients for the wavelength solution or transformation, 
indicate the model for the spectragraph for the first guess, or provide an 
image with a solution already as the first guess.  

Author                 Version      Date
-----------------------------------------------
S. M. Crawford (SAAO)    1.0       10 Oct 2009

TODO
----


LIMITATIONS
-----------
1. Currently assumes that the linelist is of the form of an ascii file with 
   either appropriate information in either one or two columns
2. Defaults to the central line

"""
# Ensure python 2.5 compatibility
from __future__ import with_statement

import os, sys
import time
import numpy as np

from pyraf import iraf 
import saltsafekey as saltkey
import saltsafeio as saltio
from saltsafelog import logging
from salterror import SaltError, SaltIOError


from PySpectrograph.WavelengthSolution import WavelengthSolution
from PySpectrograph.Models import RSSModel

import spectools as st
from spectools import SALTSpecError
from InterIdentify import InterIdentify
from AutoIdentify import AutoIdentify

debug=True



# -----------------------------------------------------------
# core routine

def specidentify(images,linelist, outfile, guesstype='rss', guessfile='',      \
                 automethod='Matchlines', function='poly', order=3, rstep=100,    \
                 rstart='middlerow', mdiff=5, thresh=3, niter=5, inter=True,   \
                 clobber=False,logfile='salt.log',verbose=True):

   with logging(logfile,debug) as log:

       #set up the variables
       infiles = []
       outfiles = []

       # Check the input images 
       infiles = saltio.argunpack ('Input',images)

       # create list of output files 
       outfiles = saltio.argunpack ('Output',outfile)

       # open the line lists
       slines, sfluxes = st.readlinelist(linelist)

       # Identify the lines in each file
       for img, ofile in zip(infiles, outfiles):
           log.message('Proccessing image %s' % img)
           if clobber and os.path.isfile(ofile): saltio.delete(ofile)
           identify(img, ofile, slines, sfluxes, guesstype=guesstype, guessfile=guessfile, 
                    function=function, order=order, rstep=rstep,  mdiff=mdiff, thresh=thresh, niter=niter,
                    method=automethod, rstart=rstart, inter=inter, log=log, verbose=verbose)


#------------------------------------------------------------------
# Find the solution for lines in a file

def identify(img, outfile, slines, sfluxes, guesstype='rss', guessfile='', function='poly', order=3,  
             rstep=1, nrows=1, mdiff=5, thresh=3, niter=5, dc=3, ndstep=50, 
             method='Zeropoint', rstart='middlerow', inter=True, log=None, verbose=True):
   """For a given image, find the solution for each row in the file.  Use the appropriate first guess and 
      guess type along with the appropriate function and order for the fit.

      Write out the new image with the solution in the headers and/or as a table in the multi-extension
      fits file
   
      img--the input fits file
      outfile--file to write the results to
      slines--wavelengths for known lines
      sfluxes--fluxes for known lines
      guesstype--type of initial guess for the solution
      guessfile--File if guesstype is from a previously processed source
      function--function form fit 
      order--order of the fit
      rstep--rows to step when fitting the solution
      nrows--number of rows to average when extracting a line
      mdiff--limit in pixels for matching
      thresh--threshhold for detecting lines and for rejecting sources
      niter--maximum number of iterations to make
      dc--step size for zero point cross correlation
      dstep--number of 
      method--method for automatic wavelength determination
      rstart--first line to extract  *TODO*
      inter--run in interactive mode
      verbose--print out the results

      returns 
   """
   ImageSolution={}
   logmsg=''

   #Open up the image
   hdu=saltio.openfits(img)   


   #determine the central row and read it in
   try:
       data=hdu[1].data
       istart=int(0.5*len(data))
       xarr=np.arange(len(data[istart]), dtype='int64')
       specarr=data
   except Exception, e:
       message = 'Unable to read in data array in %s because %s' % (img, e)
       raise SALTSpecError(message)


   #determine the type of first guess.  Assumes none 
   if guesstype=='rss':
       dateobs=saltkey.get('DATE-OBS', hdu[0], img)
       try:
           utctime=saltkey.get('UTC-OBS', hdu[0], img)
       except SaltError:
           utctime=saltkey.get('TIME-OBS', hdu[0], img)

       instrume=saltkey.get('INSTRUME', hdu[0], img).strip()
       grating=saltkey.get('GRATING', hdu[0], img).strip()
       grang=saltkey.get('GR-ANGLE', hdu[0], img)
       arang=saltkey.get('AR-ANGLE', hdu[0], img)
       filter=saltkey.get('FILTER', hdu[0], img)
       slitname=saltkey.get('MASKID', hdu[0], img)
       slit=st.getslitsize(slitname)
       xbin, ybin = saltkey.ccdbin( hdu[0], img)
       if not instrume in ['PFIS', 'RSS']:
           msg='%s is not a currently supported instrument' % instrume
           raise SALTSpecError(msg)
       #set up the rss model
       rss=RSSModel.RSSModel(grating_name=grating.strip(), gratang=grang, \
                              camang=arang,slit=slit, xbin=xbin, ybin=ybin)
       res=1e7*rss.calc_resolelement(rss.alpha(), -rss.beta())
       dres=res/10.0
       wcen=1e7*rss.calc_centralwavelength()
       R=rss.calc_resolution(wcen/1e7, rss.alpha(), -rss.beta())
       logmsg += '\nParameters for %s\nGrating\tGR-ANGLE\tAR-ANGLE\tSlit\tWCEN\tR\n' % img
       logmsg += '%s\t%8.3f\t%8.3f\t%4.2f\t%6.2f\t%4f\n' % (grating, grang, arang, slit, wcen, R)
       if log: log.message(logmsg, with_header=False)

       ws=st.useRSSModel(xarr, rss, function=function, order=order)
   elif guesstype=='user':
       raise SALTSpecError('This guesstype is not currently supported')
   elif guesstype=='image':
       raise SALTSpecError('This guesstype is not currently supported')
   elif guesstype=='None':
       raise SALTSpecError('This guesstype is not currently supported')
   else:
       ws=None


   #update some of the parameters of the wavelength solution
   if ws is not None:
      ws.thresh=thresh
      ws.niter=niter 
 
   #run in either interactive or non-interactive mode
   if inter:
       ImageSolution=InterIdentify(xarr, specarr, slines, sfluxes, ws, mdiff=mdiff, rstep=rstep, 
                           function=function, order=order, sigma=thresh, niter=niter, 
                           res=res, dres=dres, dc=dc, ndstep=ndstep, istart=istart,
                           method=method, filename=img, verbose=True)
   else:
       ws=AutoIdentify(xarr, specarr, slines, sfluxes, ws,  \
                       rstep=rstep, method=method, istart=istart, nrows=nrows, oneline=True, \
                       res=res, dres=dres, mdiff=mdiff, dc=dc, ndstep=ndstep, sigma=thresh, niter=niter, \
                       verbose=verbose) 

       ImageSolution = AutoIdentify(xarr, specarr, slines, sfluxes, ws,  \
                         rstep=rstep, method=method, istart=istart, nrows=nrows, \
                         res=res, dres=dres, dc=dc, ndstep=ndstep, sigma=thresh, niter=niter, \
                         verbose=verbose)


   #set up the list of solutions to into an array
   key_arr=np.array(ImageSolution.keys())
   arg_arr=key_arr.argsort()
   ws_arr=np.zeros((len(arg_arr),len(ws.coef)+1), dtype=float)

   #write the solution to an array 
   for j,i in enumerate(arg_arr):
      if  isinstance(ImageSolution[key_arr[i]], WavelengthSolution.WavelengthSolution):
          function=ImageSolution[key_arr[i]].function
          order=ImageSolution[key_arr[i]].order
          ws_arr[j,0]=key_arr[i]
          ws_arr[j,1:]=ImageSolution[key_arr[i]].coef

   #write the solution as an file 
   if outfile:
       #write header to the file that should include the order and function
       if os.path.isfile(outfile):
          dout=open(outfile, 'a')
       else:
          dout=open(outfile, 'w')

       msg='#WS: Wavelength solution for image %s\n' % img
       msg+= '#The following parameters were used in determining the solution:\n'
       msg+= '#name=%s\n' % img 
       msg+= '#time-obs=%s %s\n' % (dateobs, utctime )
       msg+= '#instrument=%s\n' % instrume
       msg+= '#grating=%s\n' % grating.strip()
       msg+= '#graang=%s\n' % grang   
       msg+= '#arang=%s\n' % arang 
       msg+= '#filter=%s\n' % filter.strip() 
       msg+= '#Function=%s\n' % function
       msg+= '#Order=%s\n' % order
       msg+= '#Starting Data\n'
       dout.write(msg)

       for i in range(len(ws_arr)):
           if ws_arr[i,0]:
               msg = '%5.2f ' % ws_arr[i,0]
               msg +=' '.join(['%e' % k for k in ws_arr[i,1:]])
               dout.write(msg+'\n')
       dout.write('\n')
       dout.close()
          

   #write the solution as an extension
   hdu.close()

   return 



# main code 

parfile = iraf.osfn("saltspec$specidentify.par") 
t = iraf.IrafTaskFactory(taskname="specidentify",value=parfile,function=specidentify, pkgname='saltspec')
