#!/usr/bin/env python
################################# LICENSE ##################################
# Copyright (c) 2009, South African Astronomical Observatory (SAAO)        #
# All rights reserved.                                                     #
#                                                                          #
# Redistribution and use in source and binary forms, with or without       #
# modification, are permitted provided that the following conditions       #
# are met:                                                                 #
#                                                                          #
#     * Redistributions of source code must retain the above copyright     #
#       notice, this list of conditions and the following disclaimer.      #
#     * Redistributions in binary form must reproduce the above copyright  #
#       notice, this list of conditions and the following disclaimer       #
#       in the documentation and/or other materials provided with the      #
#       distribution.                                                      #
#     * Neither the name of the South African Astronomical Observatory     #
#       (SAAO) nor the names of its contributors may be used to endorse    #
#       or promote products derived from this software without specific    #
#       prior written permission.                                          #
#                                                                          #
# THIS SOFTWARE IS PROVIDED BY THE SAAO ''AS IS'' AND ANY EXPRESS OR       #
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           #
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   #
# DISCLAIMED. IN NO EVENT SHALL THE SAAO BE LIABLE FOR ANY                 #
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       #
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS  #
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)    #
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,      #
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          #
# POSSIBILITY OF SUCH DAMAGE.                                              #
############################################################################
"""
SPECRECTIFY is a program to read in SALT RSS spectroscopic data and 
a wavelength solution for that data.  It will then apply that wavelength
solution to the data set.

For the solution, the user should be able to specify a model, an ascii file, a fits 
file, or a set of files.  The ascii file also might have multiple solutions  
in it or the program might be given a set of fits files.  It should look 
through the solutions and datetime of the solutions and create a solution
which is the interpolation over that time.  

Author                 Version      Date
-----------------------------------------------
S. M. Crawford (SAAO)    1.0       01 May 2010

TODO
----
1. Implement the fits files for wavelength solution
2. Implement multiple solutions/data


LIMITATIONS
-----------

1.  A mix of ascii and fits files are not supported

2.  The size of the output image is set by the input image

3.  Pixel scale is currently hardwired for RSS

4.  Linear interpolations between steps in the wavelength solution

"""
# Ensure python 2.5 compatibility
from __future__ import with_statement

import os, string, sys, glob, pyfits, math, datetime
import numpy as np

from pyraf import iraf 
import saltsafekey as saltkey
import saltsafeio
from saltsafelog import logging, SaltLog


from PySpectrograph.WavelengthSolution import WavelengthSolution
from PySpectrograph.Models import RSSModel

import spectools as st
from spectools import SALTSpecError


debug=True
import pylab as pl


# -------------------------------------------------------------------------
# core routine

def specrectify(images,outimages, outpref, solfile, caltype='line',            \
                function='polynomial', order=3, inttype='linear', w1=None,     \
                w2=None, dw=None, nw=None, blank=0, clobber=True,              \
                logfile='salt.log',verbose=True):

   with logging(logfile,debug) as log:

       #set up the variables
       infiles = []
       outfiles = []
       solfiles=[]

       # Check the input images 
       infiles = saltsafeio.argunpack ('Input',images)

       #read in solution file
       solfiles = saltsafeio.argunpack ('Solfile',solfile)

       # create list of output files 
       outfiles=saltsafeio.listparse('Outimages', outimages, outpref,infiles,'')

       #read in the wavelength solutions and enter them into 
       #there own format
       if caltype=='line':
           soldict=entersolution(solfiles)
       else:
          soldict=None

       #read in rectify each image
       for img, oimg, in zip(infiles, outfiles):
           if caltype=='line':
               msg='Creating rectified image %s from image %s using files %s' % (oimg, img, solfile)
           else:
               msg='Creating rectified image %s from image %s using RSS Model' % (oimg, img)
           log.message(msg)
           hdu=saltsafeio.openfits(img)
           hdu=rectify(hdu, soldict, caltype=caltype, function=function, 
                   order=order, inttype=inttype, w1=w1, w2=w2, dw=dw, nw=nw, 
                   pixscale=0.0, blank=blank, clobber=clobber, verbose=verbose)
           #write out the oimg
           saltsafeio.writefits(hdu, oimg, clobber=clobber)


# -----------------------------------------------------------
# rectify data 

def rectify(hdu, soldict, caltype='line', function='poly', order=3, inttype='interp', 
            w1=None, w2=None, dw=None, nw=None,
            blank=0, pixscale=0.0, time_interp=False, clobber=True, verbose=True):
   """Read in an image and a set of wavlength solutions.  Calculate the best
      wavelength solution for a given dataset and then apply that data set to the 
      image 

    return
   """

   #set up the time of the observation
   dateobs=saltkey.get('DATE-OBS', hdu[0])
   utctime=saltkey.get('TIME-OBS', hdu[0])
   exptime=saltkey.get('EXPTIME', hdu[0])
   instrume=saltkey.get('INSTRUME', hdu[0]).strip()
   grating=saltkey.get('GRATING', hdu[0]).strip()
   grang=saltkey.get('GR-ANGLE', hdu[0])
   arang=saltkey.get('AR-ANGLE', hdu[0])
   filtername=saltkey.get('FILTER', hdu[0]).strip()
   slitname=saltkey.get('MASKID', hdu[0])
   slit=st.getslitsize(slitname)
   xbin, ybin = saltkey.ccdbin( hdu[0])

   timeobs=enterdatetime('%s %s' % (dateobs, utctime))

   #set up the xarr and initial wavlength solution
   #open up the data
   for i in range(1,len(hdu)):
     if hdu[i].name=='SCI':
       istart=int(0.5*len(hdu[i].data))
       xarr=np.arange(len(hdu[i].data[istart]), dtype='int64')

       #set up a wavelength solution
       w_arr=findsol(xarr, soldict, istart, caltype, timeobs, exptime, instrume, grating, grang, arang, filtername, 
              slit, xbin, ybin, function, order)
       #set up the output x-axis

       if w1==None: w1=w_arr.min()
       if w2==None: w2=w_arr.max()
       if nw==None: nw=len(xarr)
       if dw==None: dw=float(w2-w1)/nw
       nw_arr=createoutputxaxis(w1, w2, nw)

       #setup the VARIANCE and BPM frames       
       if saltkey.found('VAREXT', hdu[i]):
           varext=saltkey.get('VAREXT', hdu[i])
       else:
           varext=None

       #setup the BPM frames       
       if saltkey.found('BPMEXT', hdu[i]):
           bpmext=saltkey.get('BPMEXT', hdu[i])
       else:
           bpmext=None

       #for each line in the data, determine the wavelength solution
       #for a given line in the image
       for j in range(len(hdu[i].data)):
 
           #find the wavelength solution for the data
           w_arr=findsol(xarr, soldict, j, caltype, timeobs, exptime, instrume, grating, grang, arang, filtername, 
              slit, xbin, ybin, function, order)

           #apply that wavelength solution to the data
           if w_arr is not None:
               try:
                  hdu[i].data[j,:]=st.interpolate(nw_arr, w_arr, hdu[i].data[j,:], inttype, left=blank, right=blank)
               except Exception, e:
                  hdu[i].data[j,:]=hdu[i].data[j,:]*0.0+blank
                  msg='In row %i, solution cannot be found due to %s' % (i, e)  

               #correct the variance frame
               if varext:
                   try:
                       hdu[varext].data[j,:]=st.interpolate(nw_arr, w_arr, hdu[varext].data[j,:], inttype, left=blank, right=blank)
                   except Exception, e:
                       msg='In row %i, solution cannot be found due to %s' % (i, e)  

               #correct the BPM frame
               if bpmext:
                   try:
                       hdu[bpmext].data[j,:]=st.interpolate(nw_arr, w_arr, hdu[bpmext].data[j,:], inttype, left=blank, right=blank)
                   except Exception, e:
                       msg='In row %i, solution cannot be found due to %s' % (i, e)  
           else:
               hdu[i].data[j,:]=hdu[i].data[j,:]*0.0+blank


       #Add WCS information
       saltkey.new('CTYPE1', 'LAMBDA', 'Coordinate Type', hdu[i])
       saltkey.new('CTYPE2', 'PIXEL', 'Coordinate Type', hdu[i])
       saltkey.new('CD1_1', dw, 'WCS: Wavelength Dispersion in angstrom/pixel', hdu[i])
       saltkey.new('CD2_1', 0.0, 'WCS: ', hdu[i])
       saltkey.new('CD1_2', 0.0, 'WCS: ', hdu[i])
       saltkey.new('CD2_2', ybin*pixscale, 'WCS: ', hdu[i])
       saltkey.new('CRPIX1', 0.0, 'WCS: X Reference pixel', hdu[i])
       saltkey.new('CRPIX2', 0.0, 'WCS: Y Reference pixel', hdu[i])
       saltkey.new('CRVAL1',  w1, 'WCS: X Reference pixel', hdu[i])
       saltkey.new('CRVAL2', 0.0, 'WCS: Y Reference pixel', hdu[i])
       saltkey.new('CDELT1', 1.0, 'WCS: X pixel size', hdu[i])
       saltkey.new('CDELT2', 1.0, 'WCS: Y pixel size', hdu[i])

   return hdu

def findsol(xarr, soldict, y_i, caltype, timeobs, exptime, instrume, grating, grang, arang, filtername,
              slit, xbin, ybin, function, order):
   """Find the wavelength solution.  Either from a database containing all the 
      solutions adn calculate the best one or calculate based on the model
      for RSS
   """
 

   if caltype=='line':
      function, order, coef=findlinesol( soldict, y_i, timeobs, exptime, instrume, grating, grang, arang, filtername, xarr)
      if function is None: 
          msg='No solution matches for %s, %s, %s, %s, %s' % (instrume, grating, grang, arang, filtername)
          raise SALTSpecError(msg)
      if coef is None: return None
      order=int(order)
      ws=WavelengthSolution.WavelengthSolution(xarr, xarr, function=function, order=order)
      ws.set_coef(coef)
      w_arr=ws.value(xarr)
     
   elif caltype=='rss':
      w_arr=calcsol(xarr, y_i, instrume, grating, grang, arang, filtername, slit, xbin, ybin, function, order)
   else:
      message='SALTRECTIFY--Invalid caltype'
      raise SALTSpecError(message)

   return w_arr

# -----------------------------------------------------------------------------
# set the output axis wavelenght values 

def createoutputxaxis(wstart, wend, nw):
   """Create the output array for the axis for the data to be transformed onto.  
   """
   
   try:
       warr=np.linspace(wstart, wend, num=nw)
   except Exception, e:
       msg='No output array was create because %s' %e
       raise SALTSpecError(msg)
   return warr

def calcsol(xarr, y_i, instrume, grating, grang, arang, filtername, slit, xbin, ybin, function, order):

   rss=RSSModel.RSSModel(grating_name=grating.strip(), gratang=grang, \
                              camang=arang,slit=slit, xbin=xbin, ybin=ybin)
   gamma=180.0/math.pi * math.atan((y_i*rss.detector.pix_size*rss.detector.ybin  \
               -0.5*rss.detector.find_height())/rss.camera.focallength)
   #y_i/rssmodel.detector.height*rssmodel
   d=rss.detector.xbin*rss.detector.pix_size*(xarr-rss.detector.get_xpixcenter())
   alpha=rss.gratang
   beta=rss.gratang-rss.camang
   dbeta=-np.degrees(np.arctan(d/rss.camera.focallength))
   w_arr=1e7*rss.calc_wavelength(alpha, beta+dbeta, gamma=gamma)

   return w_arr
    


# -----------------------------------------------------------------------------
# Find the best wavelength solution

def findlinesol(soldict, yc, timeobs, exptime, instrume, grating, grang, arang, filtername, xarr=None):
    """Find the best wavelength solution given a datetime of the observation
       and a wavelenght line.  We will assume that the best solution is found
       between the two solutions which are on either side of the timeobs or 
       otherwise the closest time.  

    """

    #first find the closest wavelength solution in time and use that for
    #a base sample
    time_list=[]
    function_list=[]
    order_list=[]
    coef_list=[]
    for sol in soldict:
        if matchobservations(soldict[sol], instrume, grating, grang, arang, filtername):
           #right at the same time
           if (soldict[sol][0]-timeobs).seconds==0.0:
              function=soldict[sol][6]
              order=soldict[sol][7]
              coef=findcoef(yc, soldict[sol][8], soldict[sol][9])
              return function, order, coef
           else:
              t=timeobs+datetime.timedelta(seconds=exptime)
              time_list.append(subtracttime(soldict[sol][0], t))
              function_list.append(soldict[sol][6])
              order_list.append(soldict[sol][7])
              coef_list.append(findcoef(yc, soldict[sol][8], soldict[sol][9]))

    if len(time_list)==1:
       function=function_list[0]
       order=order_list[0]
       coef=coef_list[0]
    elif len(time_list)<1:
       return None, None, None
    else:
       #determine the points closest in time to the observation
       t_arr=np.array(time_list)
       id_min=t_arr.argmin()
       function=function_list[id_min]
       order=order_list[id_min]
       coef=coef_list[id_min]

       #If xarr is None, return the solution closest in time to the obsevation
       if coef is None: return function, order, coef

       #If xarr is None, return the solution closest in time to the obsevation
       if xarr is None: return function, order, coef
       #If xarr, then calculation w_arr for each of the solutions available,
       #After calculating the solution, calculated the time weighted average
       #wavelength calibration for that pixel array.  Then recalculate the solution
       #using the same funcitonal form as the best function  
       w_ave=xarr*0.0
       wei=0.0
       for i,t in enumerate(time_list):
          if t==0: return function, order, coef
          ws=WavelengthSolution.WavelengthSolution(xarr, xarr, function=function_list[i], order=order_list[i])
          ws.set_coef(coef_list[i])
          try:
              w_ave += ws.value(xarr) / t**2
              wei += 1.0/t**2   
          except:
              pass
       w_ave = w_ave/wei
       ws=WavelengthSolution.WavelengthSolution(xarr, w_ave, function=function, order=order)
       ws.fit()
       coef=ws.coef

    return function,order,coef

# -----------------------------------------------------------------------------
# Determine if two observations are the same

def matchobservations(sol, instrume, grating, grang, arang, filtername):
    if sol[1]!=instrume: return False
    if sol[2]!=grating: return False
    if abs(sol[3]-grang)>0.01: return False
    if abs(sol[4]-arang)>0.01: return False
    if sol[5]!=filtername: return False
    return True

# -----------------------------------------------------------------------------
# Find the best coefficient solution

def findcoef(yc, wsrow, wscoef):
   """Find the coefficient given a row.  If yc can be found in wsrow, then that 
      coefficient is returned.  Otherwise, it will linearly interpoloate between
      the rows and find the best coefficient
   """
   #If it is off the edges, then return None
   if yc< wsrow.min(): return None
   if yc> wsrow.max(): return None

   #if it is in the list, then return that coef
   dist=abs(wsrow-yc)
   if dist.min()==0: 
      pass
      #return wscoef[dist.argmin()]

   #if it isn't in the list, then compute by taking
   #the interpolation between the two end points
   coefarr=np.array(wscoef)
   order=len(coefarr[0])
   coef=np.zeros(order, dtype=float)
   #print yc,
   for k in range(order):
       tk=np.polyfit(wsrow, coefarr[:,k], 2)
       coef[k]=np.polyval(tk, yc)
       
       #if k==3: print coef[k],
       #coef[k]=st.interpolate(yc, wsrow, coefarr[:,k], type='spline')
       #if k==3: print coef[k],
   #print 

   return coef
   


# -----------------------------------------------------------------------------
# Enter in the solution data


def entersolution(solfiles):
   """For each solution in the list, enter the information about that solution
      into the file.  The following information will be entered in:
      name, time, grating, grating angle, camera angle, function, order, solution
    
      return dict
   """
   soldict={}

   for solfile in solfiles:
       if solfile[-4:]=='fits':
          print 'Not supported yet'
       else:
          soldict=readsolascii(solfile, soldict)

   return soldict

def readsolascii(solfile, soldict):
   """Read in an ascii file that has the wavelength solution in formation in it.  
      The ascii file is expected to be in the same format as the output from 
      saltidentify 

      return soldict
   """
   #open up the ascii file
   fin=saltsafeio.openascii(solfile, 'r')
   data=fin.read()

   #Each new solution in the file should have WS at the start of it.  We will split
   #the file on ws
   sol_list=data.split('#WS')
   for sol in sol_list[1:]: 
       name=findwskeyword('name', sol)
       timeobs=enterdatetime(findwskeyword('time-obs', sol))
       instrume=findwskeyword('instrument', sol)
       grating=findwskeyword('grating', sol)
       graang=float(findwskeyword('graang', sol))
       arang=float(findwskeyword('arang', sol))
       filtername=findwskeyword('filter', sol)
       function=findwskeyword('Function', sol)
       order=int(findwskeyword('Order',sol))
       wsrow=[]
       wscoef=[]
       j=finddata(sol)
       for ws  in sol[j:].splitlines():
           if ws.strip():
               ws=ws.split()
               wsrow.append(float(ws[0]))
               wscoef.append(np.array(ws[1:], dtype=float))
       soldict[name]=[timeobs, instrume, grating, graang, arang, filtername, function, order, np.array(wsrow), wscoef]
   
   return soldict

def finddata(sol):
    """Find where the data starts in the solution"""
    i=sol.index('#Starting Data')
    j=sol[i:].index('\n')
    return i+j+1
    

def findwskeyword(keyword, sol):
    """Find and return a value for a keyword in the list of the wavelength solution"""
    i=sol.index(keyword)
    j=sol[i:].index('\n')
    return sol[i:i+j].split('=')[1].strip()

def enterdatetime(dstr):
    """Break up the datetime string to create a datetime object
       return datetime
    """
    dlist=dstr.split() 
    year, month, day=dlist[0].split('-')
    hour, minute, second = dlist[1].split(':')
    
    return datetime.datetime(int(year), int(month), int(day), int(hour), int(minute),float(second))

def subtracttime(d1, d2):
    """Return the difference in two dates in seconds"""
    dt=max(d1,d2)-min(d1,d2)
    return 86400*dt.days+dt.seconds


# main code 

parfile = iraf.osfn("saltspec$specrectify.par") 
t = iraf.IrafTaskFactory(taskname="specrectify",value=parfile,function=specrectify, pkgname='saltspec')
